
PROJECT_NAME := $(notdir $(abspath .))

include ../../../rules/env.make

include $(LANTRONIX_DIRECTORY)/version.make

export FEATURE_FLAGS :=
-include defines.make

include modules.make

export PROJECT_INCLUDES := \
    $(LANTRONIX_DIRECTORY)/includes/system \
    $(LANTRONIX_DIRECTORY)/includes \
    $(BASE_DIRECTORY)/includes \
    $(WORK_DIRECTORY)/$(PROJECT_NAME)/lantronix/includes \
    $(WORK_DIRECTORY)/$(PROJECT_NAME)/includes \

TARGET_NAME := $(PROJECT_NAME)_$(LANTRONIX_VERSION)_$(SDK_VERSION)

OBJ_DIR := $(WORK_DIRECTORY)/$(PROJECT_NAME)/objects
DEP_DIR := $(WORK_DIRECTORY)/$(PROJECT_NAME)/dependencies

.SUFFIXES:

.PHONY: all
.DEFAULT_GOAL := all

CROSS := $(TOOLCHAIN_BIN_DIR)/arm-none-eabi-
CC := $(CROSS)gcc
LD := $(CROSS)ld
AR := $(CROSS)ar
AS := $(CROSS)as
STRIP := $(CROSS)strip
OBJCOPY := $(CROSS)objcopy
OBJDUMP := $(CROSS)objdump
SIZE := $(CROSS)size
ifeq ($(HOST_PLATFORM),Linux)
MKIMAGE := $(LANTRONIX_DIRECTORY)/bin/ltrx-mkimage-linux64
else
MKIMAGE := $(LANTRONIX_DIRECTORY)/bin/ltrx-mkimage.exe
endif

CFLAGS = -std=c99 -ffunction-sections -fdata-sections

WARNFLAGS = \
  -Wall \
  -Wextra \
  -Wno-missing-field-initializers \
  -Wno-array-bounds \
  -Wmissing-include-dirs \
  -Wunused-parameter \
  -Wunknown-pragmas \
  -Wundef \
  -Wshadow \
  -Wpointer-arith \
  -Wcast-qual \
  -Wcast-align \
  -Wwrite-strings \
  -Wstrict-prototypes \
  -Wold-style-declaration \
  -Wmissing-prototypes \
  -Wmissing-declarations \
  -Wredundant-decls \
  -Wnested-externs \
  -Winline \
  -Winvalid-pch \

WARNFLAGS += -Werror

ASMFLAGS =

LDFLAGS = -static -nostartfiles -Wl,--cref -Wl,--warn-common -Wl,-d

SYSTEM_INCLUDES := \
  $(TOOLCHAIN_BASE_DIR)/include \
  $(TOOLCHAIN_BASE_DIR)/lib/include \
  $(TOOLCHAIN_BASE_DIR)/lib/include-fixed \

.PHONY: debug
debug: all

ifneq ($(filter debug, $(MAKECMDGOALS)),)
CFLAGS += -DDEBUG -ggdb -O0
ASMFLAGS += --defsym DEBUG=1 -ggdb
LDFLAGS  +=
BUILD_TYPE = debug
else
CFLAGS += -ggdb -Os
ASMFLAGS +=
LDFLAGS  += -Wl,-Os
BUILD_TYPE = release
endif

BUILD_OPTS := \
  CC="$(CC) $(addprefix -isystem ,$(SYSTEM_INCLUDES))" \
  CFLAGS="$$(CFLAGS)" \
  AS="$(AS)" \
  ASMFLAGS="$$(ASMFLAGS)" \
  AR=$(AR) \
  LD=$(LD) \
  OBJCOPY=$(OBJCOPY) \
  MKDIR=$(MKDIR) \
  BASE_DIRECTORY="$(BASE_DIRECTORY)" \
  PROJECT_NAME=$(PROJECT_NAME) \
  BUILD_TYPE=$(BUILD_TYPE) \

$(WORK_DIRECTORY)/$(PROJECT_NAME)/project/.makefile: \
  Makefile $(wildcard defines.make) modules.make $(RULES_DIRECTORY)/common.make
	@echo $@
	@$(MKDIR) -p $(addprefix $(WORK_DIRECTORY)/$(PROJECT_NAME)/, \
	  lantronix \
	  lantronix/includes \
	  lantronix/libraries \
	  lantronix/modules \
	  dependencies \
	  includes \
	  libraries \
	  modules \
	  objects \
	  project \
	)
	@touch $@

.PHONY: clean
clean:
	$(RM) -r $(WORK_DIRECTORY)/$(PROJECT_NAME)

ALL_C_COMPILE_SOURCES :=
ALL_ASM_SOURCES :=

FEATURE_FLAG_VALUE = $(word 2,$(subst =, ,$(filter $(1)=%,$(FEATURE_FLAGS))))

export EVALUATED_FLAGS :=

BARE_C_DEFINES := $(strip $(FEATURE_FLAGS) $(EVALUATED_FLAGS))

C_DEFINES := $(addprefix -D,$(BARE_C_DEFINES))

ALL_MODULE_DIRECTORIES := $(dir $(wildcard $(BASE_DIRECTORY)/module/*/bom/cfiles.make))

ALL_MODULE_NAMES := $(filter-out $(EXCLUDED_MODULES),$(foreach DIR,$(ALL_MODULE_DIRECTORIES),$(lastword $(subst /, ,$(patsubst %/bom/,%,$(DIR))))))

ALL_UNSORTED_MODULE_DEFS := \
  $(foreach \
    MODULE, \
    $(ALL_MODULE_NAMES), \
    $(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(strip $(MODULE))/$(strip $(MODULE))_module_defs_unsorted.dat \
  )

$(WORK_DIRECTORY)/$(PROJECT_NAME)/module_defs_sort.txt : $(ALL_UNSORTED_MODULE_DEFS)
	@echo $(notdir $@)
	@cat $(ALL_UNSORTED_MODULE_DEFS) | sort -r -g -u | unix2dos > $@

PRUNE_TO_MODULE_DIRECTORY = $(patsubst $(BASE_DIRECTORY)/module/%,%,$(1))

define COPY_template
$(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/embedded/files/$(call PRUNE_TO_MODULE_DIRECTORY,$(2)): $(2)
	@echo $$<
	@$(MKDIR) -p $$(dir $$@)
	@cp -f $$< $$@
endef

define PREPROCESS_template
$(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/embedded/files/$(call PRUNE_TO_MODULE_DIRECTORY,$(2)): \
  $(2) $(WORK_DIRECTORY)/$(PROJECT_NAME)/project/.makefile
	@echo $$<
	@$(MKDIR) -p $$(dir $$@)
	@$(CC) -E -P -x c -C -Wundef -Werror $(C_DEFINES) $$< | sed -e 's/^[ \t]*//g' | sed -e 's/&apos;/\x27/g' > $$@
endef

define GZIP_template
$(1)_EMBEDDED_FILES = $(subst $(2),$(2).gz,$($(1)_EMBEDDED_FILES))
$(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/embedded/files/$(call PRUNE_TO_MODULE_DIRECTORY,$(2)).gz: $(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/embedded/files/$(call PRUNE_TO_MODULE_DIRECTORY,$(2))
	@gzip -c $$< > $$<.gz
endef

define BUILD_FILE_STRUCT_template
$(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/embedded/c/$(call PRUNE_TO_MODULE_DIRECTORY,$(2)).c: \
  $(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/embedded/files/$(call PRUNE_TO_MODULE_DIRECTORY,$(2))
	@echo $$<
	@$(MKDIR) -p $$(dir $$@)
	@echo "/* Generated by makefile */" > $$@
	@echo "#include \"ltrx_embedded_files.h\"" >> $$@
	@echo "const struct ltrx_embedded_file Embedded_file_$(subst /,_slash_,$(subst .,_dot_,$(call PRUNE_TO_MODULE_DIRECTORY,$(2))))_contents = {" >> $$@
	@echo "\"$(call PRUNE_TO_MODULE_DIRECTORY,$(2))\"", >> $$@
	@echo "`stat -c%s $$<`," >>$$@
	@od -An -bv $$< | sed -e 's/^/"/g' -e 's/$$$$/"/g' -e 's/ /\\/g' >> $$@
	@echo -e "\"\"\n};" >> $$@
endef

define MODULE_DEFINES_template

MY_DIR := $(BASE_DIRECTORY)/module/$(1)

$(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/$(1)_module_defs_unsorted.dat : \
  $(RULES_DIRECTORY)/module_defs_sort.m4 \
  $(RULES_DIRECTORY)/module_defs_clean.m4 \
  $(wildcard $(BASE_DIRECTORY)/module/$(1)/module_defs.m4d) \
  $(WORK_DIRECTORY)/$(PROJECT_NAME)/project/.makefile
	@echo $$(notdir $$@)
	@$(MKDIR) -p $$(dir $$@)
	@m4 \
	  -I$(BASE_DIRECTORY)/module/$(1) \
	  -I$(RULES_DIRECTORY) \
	  $(addprefix -D,$(strip MODULE_NAME=$(1) $(FEATURE_FLAGS) $(EVALUATED_FLAGS))) \
	  $$< > $$@

$(WORK_DIRECTORY)/$(PROJECT_NAME)/lantronix/includes/$(1)_module_definitions.h : \
  $(RULES_DIRECTORY)/module_definitions_h.m4 \
  $(RULES_DIRECTORY)/module_defs_clean.m4 \
  $(wildcard $(BASE_DIRECTORY)/module/$(1)/module_defs.m4d) \
  $(WORK_DIRECTORY)/$(PROJECT_NAME)/project/.makefile
	@echo $$(notdir $$@)
	@m4 \
	  -I$(BASE_DIRECTORY)/module/$(1) \
	  -I$(RULES_DIRECTORY) \
	  -DTHIS_MODULE_NAME=\`$(1)\' \
	  $(addprefix -D,$(strip $(FEATURE_FLAGS) $(EVALUATED_FLAGS))) \
	  $$< > $$@

$(WORK_DIRECTORY)/$(PROJECT_NAME)/lantronix/includes/$(1)_module_libs.h : \
  $(RULES_DIRECTORY)/module_libs_h.m4 \
  $(RULES_DIRECTORY)/module_defs_clean.m4 \
  $(wildcard $(BASE_DIRECTORY)/module/$(1)/module_defs.m4d) \
  $(WORK_DIRECTORY)/$(PROJECT_NAME)/project/.makefile
	@echo $$(notdir $$@)
	@m4 \
	  -I$(BASE_DIRECTORY)/module/$(1) \
	  -I$(RULES_DIRECTORY) \
	  -DTHIS_MODULE_NAME=\`$(1)\' \
	  $(addprefix -D,$(strip $(FEATURE_FLAGS) $(EVALUATED_FLAGS))) \
	  $$< > $$@

$(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/$(1)_module_libs.c : \
  $(RULES_DIRECTORY)/module_libs_c.m4 \
  $(RULES_DIRECTORY)/module_defs_clean.m4 \
  $(wildcard $(BASE_DIRECTORY)/module/$(1)/module_defs.m4d) \
  $(WORK_DIRECTORY)/$(PROJECT_NAME)/project/.makefile
	@echo $$(notdir $$@)
	@$(MKDIR) -p $$(dir $$@)
	@m4 \
	  -I$(BASE_DIRECTORY)/module/$(1) \
	  -I$(RULES_DIRECTORY) \
	  -DTHIS_MODULE_NAME=\`$(1)\' \
	  $(addprefix -D,$(strip $(FEATURE_FLAGS) $(EVALUATED_FLAGS))) \
	  $$< > $$@

$(WORK_DIRECTORY)/$(PROJECT_NAME)/includes/$(1)_module_defs.h : \
  $(RULES_DIRECTORY)/module_defs_h.m4 \
  $(RULES_DIRECTORY)/module_defs_clean.m4 \
  $(wildcard $(BASE_DIRECTORY)/module/$(1)/module_defs.m4d) \
  $(WORK_DIRECTORY)/$(PROJECT_NAME)/project/.makefile
	@echo $$(notdir $$@)
	@m4 \
	  -I$(BASE_DIRECTORY)/module/$(1) \
	  -I$(RULES_DIRECTORY) \
	  -DTHIS_MODULE_NAME=\`$(1)\' \
	  $(addprefix -D,$(strip $(FEATURE_FLAGS) $(EVALUATED_FLAGS))) \
	  $$< > $$@

$(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/module_defs.c : \
  $(RULES_DIRECTORY)/module_defs_c.m4 \
  $(RULES_DIRECTORY)/module_defs_clean.m4 \
  $(wildcard $(BASE_DIRECTORY)/module/$(1)/module_defs.m4d) \
  $(WORK_DIRECTORY)/$(PROJECT_NAME)/project/.makefile
	@echo $$(notdir $$@)
	@$(MKDIR) -p $$(dir $$@)
	@m4 \
	  -I$(BASE_DIRECTORY)/module/$(1) \
	  -I$(RULES_DIRECTORY) \
	  -DTHIS_MODULE_NAME=\`$(1)\' \
	  $(addprefix -D,$(strip $(FEATURE_FLAGS) $(EVALUATED_FLAGS))) \
	  $$< > $$@

$(WORK_DIRECTORY)/$(PROJECT_NAME)/lantronix/libraries/$(1)_module_libs.o : \
  $(OBJ_DIR)/$(call MANGLE,$(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/$(1)_module_libs.o)
	@echo $$(notdir $$@)
	@cp -f -T $$< $$@

BINARY_EMBEDDED :=
-include $(BASE_DIRECTORY)/module/$(1)/bom/binary_embedded.make

PREPROC_EMBEDDED :=
-include $(BASE_DIRECTORY)/module/$(1)/bom/preproc_embedded.make

GZIP_EMBEDDED :=
-include $(BASE_DIRECTORY)/module/$(1)/bom/gzip_embedded.make

$(1)_EMBEDDED_FILES := $$(BINARY_EMBEDDED) $$(PREPROC_EMBEDDED)

$$(foreach FILE,$$(GZIP_EMBEDDED),$$(eval $$(call GZIP_template,$(1),$$(FILE))))

include $(BASE_DIRECTORY)/module/$(1)/bom/cfiles.make
MODULE_C_COMPILE_SOURCES := $$(CFILES)
MODULE_C_COMPILE_SOURCES += $(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/module_defs.c
MODULE_C_COMPILE_SOURCES += $(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/module_datetime.c
MODULE_C_COMPILE_SOURCES += \
  $(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/embedded_files.c \
  $$(addprefix $(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/embedded/c/, \
    $$(addsuffix .c, $$(call PRUNE_TO_MODULE_DIRECTORY,$$($(1)_EMBEDDED_FILES))) \
  )

$$(foreach FILE,$$(BINARY_EMBEDDED),$$(eval $$(call COPY_template,$(1),$$(FILE))))
$$(foreach FILE,$$(PREPROC_EMBEDDED),$$(eval $$(call PREPROCESS_template,$(1),$$(FILE))))
$$(foreach FILE,$$($(1)_EMBEDDED_FILES),$$(eval $$(call BUILD_FILE_STRUCT_template,$(1),$$(FILE))))

MODULE_ASM_SOURCES := $$(ASMFILES)

ALL_C_COMPILE_SOURCES += \
  $$(MODULE_C_COMPILE_SOURCES) \
  $(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/$(1)_module_libs.c

ALL_ASM_SOURCES += $$(MODULE_ASM_SOURCES)

$(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/embedded_files.c: \
  $(wildcard $(BASE_DIRECTORY)/module/$(1)/bom/binary_embedded.make) \
  $(wildcard $(BASE_DIRECTORY)/module/$(1)/bom/preproc_embedded.make) \
  $(wildcard $(BASE_DIRECTORY)/module/$(1)/bom/gzip_embedded.make) \
  $(WORK_DIRECTORY)/$(PROJECT_NAME)/project/.makefile
	@echo $$(basename $$@)
	@$(MKDIR) -p $$(dir $$@)
	@echo "/* Generated by makefile */" > $$@
	@echo "#include \"ltrx_embedded_files.h\"" >> $$@
	@echo >> $$@
	@echo "#define EMBEDDED_FILE_REFERENCE(x) Embedded_file_ ## x ## _contents" >> $$@
	@echo >> $$@
	@for f in $$(subst /,_slash_,$$(subst .,_dot_,$$(call PRUNE_TO_MODULE_DIRECTORY,$$($(1)_EMBEDDED_FILES)))); do \
	    echo "$$$$f" ; \
	done | sort | awk -v vpath="" -f $(RULES_DIRECTORY)/embedded_files1.awk >> $$@
	@echo >> $$@
	@echo "const struct ltrx_embedded_file * const g_Embedded_file_contents[] = {" >> $$@
	@for f in $$(subst /,_slash_,$$(subst .,_dot_,$$(call PRUNE_TO_MODULE_DIRECTORY,$$($(1)_EMBEDDED_FILES)))); do \
	    echo "$$$$f" ; \
	done | sort | awk -v vpath="" -f $(RULES_DIRECTORY)/embedded_files2.awk >> $$@
	@echo "    0 /* No more files */" >> $$@
	@echo "};" >> $$@


$(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/module_datetime.c : \
  $(RULES_DIRECTORY)/module_datetime.c \
  $$(CFILES) \
  $(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/module_defs.c
	@echo $$@
	@cp -T $$< $$@


$(1)_OBJECTS := \
  $$(addprefix $(OBJ_DIR)/,$$(call MANGLE,$$(MODULE_C_COMPILE_SOURCES:.c=.o))) \
  $$(addprefix $(OBJ_DIR)/,$$(call MANGLE,$$(MODULE_ASM_SOURCES:.s=.o)))

ifneq ($(wildcard $(BASE_DIRECTORY)/module/$(1)/Makefile),)
$(1)_ARCHIVE := \
  $(WORK_DIRECTORY)/$(PROJECT_NAME)/libraries/$(1)/$(1).a
else
$(1)_ARCHIVE :=
endif

LINKFORCE_SOURCES :=
-include $(BASE_DIRECTORY)/module/$(1)/linkforce.make

$(1)_linkforce_OBJECTS := $$(addprefix $(OBJ_DIR)/,$$(call MANGLE,$$(filter %.o, $$(LINKFORCE_SOURCES:.c=.o) $$(LINKFORCE_SOURCES:.s=.o))))

$(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/$(1).symbols.elf: \
  $$($(1)_ARCHIVE) \
  $$($(1)_OBJECTS) \
  $(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/main/main.elf \
  $(RULES_DIRECTORY)/module.ld
	@echo $$(notdir $$@)
	@$(CC) \
	  -r \
	  -Wl,--just-symbols=$(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/main/main.elf \
	  -Wl,--gc-sections \
	  $$(LDFLAGS) -Wl,-Map,$(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/$(1).symbols.map \
	  -Wl,-T $(RULES_DIRECTORY)/module.ld \
	  -Wl,--undefined=g_$(1)ModuleInfo \
	  -Wl,--start-group \
	  $$($(1)_linkforce_OBJECTS) \
	  $$($(1)_OBJECTS) \
	  $$(wildcard $(WORK_DIRECTORY)/$(PROJECT_NAME)/libraries/$(1)/*.a) \
	  $$(wildcard $(WORK_DIRECTORY)/$(PROJECT_NAME)/lantronix/libraries/*_module_libs.o) \
	  $$(wildcard $(LANTRONIX_DIRECTORY)/libraries/*_module_libs.o) \
	  -Wl,--end-group \
	  -o $$@

$(WORK_DIRECTORY)/$(PROJECT_NAME)/lantronix/modules/$(1).elf: \
  $(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/$(1).symbols.elf
	@echo $$(notdir $$@)
	@$(STRIP) --strip-unneeded $$< -o $$@

endef

define MODULE_SELECT_template

ifeq ($(1),$(filter $(1),$(ALL_MODULE_NAMES)))

$(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/$(1).elf: \
  $(WORK_DIRECTORY)/$(PROJECT_NAME)/lantronix/modules/$(1).elf \
  $(RULES_DIRECTORY)/module.ld \
  $(PREVIOUS_ELF_FILE)
	@echo $$(notdir $$@)
	@$(CC) \
	  $$(LDFLAGS) -Wl,-Map,$(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/$(1).map \
	  -Wl,-T $(RULES_DIRECTORY)/module.ld \
	  -Wl,--defsym,ltrx_linker_module_$(1)_flash_end=ltrx_linker_module_flash_end \
	  -Wl,--defsym,ltrx_linker_module_$(1)_ram_end=ltrx_linker_module_ram_end \
	  -Wl,--start-group \
	  -Ttext $$(call SYMBOL_VALUE,$(PREVIOUS_ELF_FILE),ltrx_linker_module_$(PREVIOUS_MODULE)_flash_end) \
	  -Tdata $$(call SYMBOL_VALUE,$(PREVIOUS_ELF_FILE),ltrx_linker_module_$(PREVIOUS_MODULE)_ram_end) \
	  $$< \
	  -Wl,--end-group \
	  -o $$@

else

ifeq ($(1),main)

$(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/$(1).elf: \
  $(LANTRONIX_DIRECTORY)/modules/$(1).elf
	@echo $$(notdir $$@)
	@$(MKDIR) -p $$(dir $$@)
	@cp -f $$< $$@

else

$(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/$(1).elf: \
  $(LANTRONIX_DIRECTORY)/modules/$(1).elf \
  $(RULES_DIRECTORY)/module.ld \
  $(PREVIOUS_ELF_FILE)
	@echo $$(notdir $$@)
	@$(MKDIR) -p $$(dir $$@)
	@$(CC) \
	  $$(LDFLAGS) -Wl,-Map,$(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/$(1).map \
	  -Wl,-T $(RULES_DIRECTORY)/module.ld \
	  -Wl,--defsym,ltrx_linker_module_$(1)_flash_end=ltrx_linker_module_flash_end \
	  -Wl,--defsym,ltrx_linker_module_$(1)_ram_end=ltrx_linker_module_ram_end \
	  -Wl,--start-group \
	  -Ttext $$(call SYMBOL_VALUE,$(PREVIOUS_ELF_FILE),ltrx_linker_module_$(PREVIOUS_MODULE)_flash_end) \
	  -Tdata $$(call SYMBOL_VALUE,$(PREVIOUS_ELF_FILE),ltrx_linker_module_$(PREVIOUS_MODULE)_ram_end) \
	  $$< \
	  -Wl,--end-group \
	  -o $$@

endif

endif

$(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/$(1).stripped.elf: \
  $(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/$(1).elf
	@echo $$(notdir $$@)
	@$(STRIP) --strip-all $$< -o $$@

$(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/$(1).bin: \
  $(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/$(1).stripped.elf
	@echo $$(notdir $$@)
	@$(OBJCOPY) -O binary -R .comment -R .ARM.attributes  $$< $$@

PREVIOUS_ELF_FILE := $(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/$(1).elf
PREVIOUS_MODULE := $(1)

endef

ALL_GENERATED_HEADERS := \
  $(foreach \
    MODULE, \
    $(ALL_MODULE_NAMES), \
	$(WORK_DIRECTORY)/$(PROJECT_NAME)/includes/$(strip $(MODULE))_module_defs.h \
    $(WORK_DIRECTORY)/$(PROJECT_NAME)/lantronix/includes/$(strip $(MODULE))_module_definitions.h \
    $(WORK_DIRECTORY)/$(PROJECT_NAME)/lantronix/includes/$(strip $(MODULE))_module_libs.h \
  )

DEPENDENCY_INCLUDES = \
  $(addprefix -isystem ,$(SYSTEM_INCLUDES)) \
  $(addprefix -iquote ,$(LOADERS_INCLUDE)) \
  $(addprefix -iquote ,$(dir $(1))) \
  $(addprefix -iquote ,$(PROJECT_INCLUDES)) \
  $(addprefix -iquote ,$(wildcard $(addsuffix /includes,$(addprefix $(BASE_DIRECTORY)/module/,$(word 1,$(subst /, ,$(patsubst $(BASE_DIRECTORY)/module/%,%,$(filter $(BASE_DIRECTORY)/module/%,$(1))))))))) \

define C_DEPENDENCY_template
$(DEP_DIR)/$(call MANGLE,$(1:.c=.dep)) : $(1) $(ALL_GENERATED_HEADERS)
	@echo $$(notdir $$@)
	@$(RM) $$@ $$@.$$$$
	@$(CC) -MM -MF '$$@.$$$$' -MQ '$$@' $(call DEPENDENCY_INCLUDES,$(1)) $(C_DEFINES) $$<
	@sed -e "s,\([a-zA-Z]\)\\:\/,\/\1\/,g" -e "s,$*[_a-z/A-Z.0-9-]*\.dep[ :]*,$(OBJ_DIR)/$(call MANGLE,$(1:.c=.o)) $$@ : ," < $$@.$$$$ > $$@
	@$(RM) $$@.$$$$
-include $(DEP_DIR)/$(call MANGLE,$(1:.c=.dep))
endef

define C_SOURCE_template
$(OBJ_DIR)/$(call MANGLE,$(1:.c=.o)) : $(1) $(ALL_GENERATED_HEADERS)
	@echo $$(notdir $$@)
	@$(CC) $$(CFLAGS) $$(WARNFLAGS) $(call DEPENDENCY_INCLUDES,$(1)) $(C_DEFINES) -c $$< -o $$@
endef

define C_SOURCE_check_template
$(1) :
endef

define ASM_SOURCE_template
$(OBJ_DIR)/$(call MANGLE,$(1:.s=.o)) : $(1)
	@echo $$(notdir $$@)
	@$(AS) $$(ASMFLAGS) -c $$< -o $$@
endef

SYMBOL_VALUE = 0x$(shell $(OBJDUMP) -t $(1) | grep $(2) | cut -c1-8)

MODULE_ORDERED_NAMES := $(filter main,$(MODULE_NAMES)) $(filter-out main,$(MODULE_NAMES))

$(WORK_DIRECTORY)/$(PROJECT_NAME)/project/project.bin: \
  $(foreach MODULE,$(MODULE_ORDERED_NAMES), \
    $(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(MODULE)/$(MODULE).bin \
  ) \
  $(WORK_DIRECTORY)/$(PROJECT_NAME)/project/.makefile
	@echo $(notdir $@)
	@$(RM) $@
	@$(foreach MODULE,$(MODULE_ORDERED_NAMES), \
		cat $(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(MODULE)/$(MODULE).bin >> $@; \
	)
	@echo >> $@

$(WORK_DIRECTORY)/$(PROJECT_NAME)/project/project.rom: \
  $(WORK_DIRECTORY)/$(PROJECT_NAME)/project/project.bin
	@echo $(notdir $@)
	@$(MKIMAGE) -m -o -a$(LOAD_ADDRESS_FW) -r$(LANTRONIX_VERSION)/$(SDK_VERSION) -i$(PRODUCT_ID) -s256 -t3 $<

$(WORK_DIRECTORY)/$(PROJECT_NAME)/$(TARGET_NAME).rom: \
  $(WORK_DIRECTORY)/$(PROJECT_NAME)/project/project.rom
	@echo $(notdir $@)
	@cp -f $< $@

cfg_targets := config cfg oemconfig oemcfg config-clean cfg-clean
.PHONY: $(cfg_targets)

config cfg oemconfig oemcfg: \
  $(WORK_DIRECTORY)/$(PROJECT_NAME)/$(CFG_TARGET_NAME).rom

$(WORK_DIRECTORY)/$(PROJECT_NAME)/$(CFG_TARGET_NAME).rom: \
  $(WORK_DIRECTORY)/$(PROJECT_NAME)/$(CFG_TARGET_NAME).bin
	@echo $(notdir $@)
	@$(MKIMAGE) -m -a$(LOAD_ADDRESS_CFG) -r$(LANTRONIX_VERSION)/$(SDK_VERSION) -t5 $<

$(WORK_DIRECTORY)/$(PROJECT_NAME)/$(CFG_TARGET_NAME).bin: $(CFG_TARGET_NAME).xml
	@echo $(notdir $@)
	@$(MKDIR) -p $(WORK_DIRECTORY)/$(PROJECT_NAME)
	@$(MKCONFIG) -qq -i $< -o $@

config-clean cfg-clean:
	@$(RM) $(WORK_DIRECTORY)/$(PROJECT_NAME)/$(CFG_TARGET_NAME).*

ifeq ($(filter clean $(cfg_targets) $(jtag_targets),$(MAKECMDGOALS)),)
$(foreach MODULE,$(ALL_MODULE_NAMES),$(eval $(call MODULE_DEFINES_template,$(strip $(MODULE)))))
$(foreach MODULE,$(MODULE_ORDERED_NAMES),$(eval $(call MODULE_SELECT_template,$(strip $(MODULE)))))
$(foreach FILE,$(ALL_C_COMPILE_SOURCES),$(eval $(call C_DEPENDENCY_template,$(FILE))))
$(foreach FILE,$(ALL_C_COMPILE_SOURCES),$(eval $(call C_SOURCE_template,$(FILE))))
$(foreach FILE,$(CFILES),$(eval $(call C_SOURCE_check_template,$(FILE))))
$(foreach FILE,$(ALL_ASM_SOURCES),$(eval $(call ASM_SOURCE_template,$(FILE))))
endif

.PHONY: image_summary
image_summary: $(PREVIOUS_ELF_FILE)
	@echo "Image Summary:"
	@echo "  Flash Usage:  0x$(shell $(OBJDUMP) -t $(PREVIOUS_ELF_FILE) | grep ltrx_linker_module_$(PREVIOUS_MODULE)_flash_end | cut -c3-8) of 0x100000"
	@echo "  SRAM Usage:   0x$(shell $(OBJDUMP) -t $(PREVIOUS_ELF_FILE) | grep ltrx_linker_module_$(PREVIOUS_MODULE)_ram_end | cut -c4-8) of  0x20000"
