HOST_PLATFORM := $(shell uname)
ifeq ($(filter MINGW% MSYS2% MSYS_NT% Linux,$(HOST_PLATFORM)),)
$(error This platform is not supported)
else
$(info Building on host platform $(HOST_PLATFORM))
endif

#
# define normalize_path based on environment.
#
ifneq ($(findstring Windows,$(OS)),)

ifneq ($(filter MINGW% CYGWIN%,$(shell uname)),)
# MinGW, Cygwin
define normalize_path
$(abspath $(1))
endef
else
# Vanilla Windows (assumes gnuwin32)
fix_slash = $(subst \,/,$(1))
strip_drive = $(filter-out %:,$(subst :,: ,$(1)))
define normalize_path
$(call strip_drive,$(abspath $(call strip_drive,$(call fix_slash,$(1)))))
endef
endif

else

# *nix
define normalize_path
$(abspath $(1))
endef

endif

PROJECT_ROOT_DIRECTORY := $(call normalize_path,../../..)
BASE_DIRECTORY := $(PROJECT_ROOT_DIRECTORY)/custom
WORK_DIRECTORY := $(PROJECT_ROOT_DIRECTORY)/work
ifeq ($(HOST_PLATFORM),Linux)
WORK_WIN_DIRECTORY := $(WORK_DIRECTORY)
else
WORK_WIN_DIRECTORY := $(shell [ -d $(WORK_DIRECTORY) ] && cd $(WORK_DIRECTORY) && pwd -W)
endif
RULES_DIRECTORY := $(PROJECT_ROOT_DIRECTORY)/rules
LANTRONIX_DIRECTORY := $(PROJECT_ROOT_DIRECTORY)/lantronix

#
# define tools based on environment.
#
ifneq ($(findstring Windows,$(OS)),)

ifneq ($(filter CYGWIN% MINGW% MSYS%,$(shell uname)),)
# Cygwin, MinGW, MSYS2
TOOLCHAIN_BASE_DIR := $(PROJECT_ROOT_DIRECTORY)/toolchain/yagarto_arm_gnu
MKDIR := mkdir
else
# Vanilla Windows (assumes gnuwin32)
$(error set Vanilla Windows build not supported at the moment)
endif

TOOLCHAIN_BIN_DIR := $(TOOLCHAIN_BASE_DIR)/bin/Win32

else

# *nix
TOOLCHAIN_BASE_DIR := $(PROJECT_ROOT_DIRECTORY)/toolchain/yagarto_arm_gnu
TOOLCHAIN_BIN_DIR := $(TOOLCHAIN_BASE_DIR)/bin/Lin64
MKDIR := mkdir

endif


ifeq ($(wildcard $(TOOLCHAIN_BASE_DIR)),)
$(info The toolchain path '$(TOOLCHAIN_BASE_DIR)' does not look correct)
$(error set $$TOOLCHAIN_BASE_DIR to point to it)
endif

SHORTENED_NAME = $(patsubst $(PROJECT_ROOT_DIRECTORY)/%,%,$(1))

MANGLE = $(subst /,__,$(subst ..,up,$(call SHORTENED_NAME,$(1))))
