
/* Lantronix minimal posix dirent interface. */

#ifndef _LTRX_DIRENT_H_
#define _LTRX_DIRENT_H_

/*****************************************************************************/
/*                             Includes                                      */
/*****************************************************************************/

#include <stdint.h>

/*****************************************************************************/
/*                           Documentation                                   */
/*****************************************************************************/

/*!
** \file
** \brief Definitions for file directory entries.
*/

/*****************************************************************************/
/*                             Defines                                       */
/*****************************************************************************/

/* Directory entry types: */

/*!
** \ingroup file_system
** \brief Directory entry type unknown.
*/
#define DT_UNKNOWN      0

/*!
** \ingroup file_system
** \brief Directory entry type directory.
*/
#define DT_DIR          1

/*!
** \ingroup file_system
** \brief Directory entry type regular file.
*/
#define DT_REG          2

#define dirfd _dirfd

/*****************************************************************************/
/*                             Structs                                       */
/*****************************************************************************/

/*!
** \ingroup file_system
** \brief Directory entry struct written by readdir_r().
*/
struct dirent
{
    /*! Size of the entry in bytes. */
    uint32_t d_size;
    /*! Type of the entry. (\c DT_REG, \c DT_DIR, or \c DT_UNKNOWN). */
    uint32_t d_type;
    /*! Name of the entry. */
    char d_name[101];
}; 

/*****************************************************************************/
/*                             Typedefs                                      */
/*****************************************************************************/

/*!
** \ingroup file_system
** \brief Directory stream type.
**
** Typedef \c DIR is a stream opened by opendir(), read by readdir_r(),
** and closed by closedir().
*/
typedef void DIR;

/*****************************************************************************/
/*                            Prototypes                                     */
/*****************************************************************************/

/*!
** \ingroup file_system
** \brief Open a directory stream.
**
** Opens a directory stream corresponding to the directory named by \a path.
** The directory stream is positioned at the first entry.
**
** \returns Pointer to an object of type \c DIR.
** \retval NULL Error, see \c errno.
*/
DIR *opendir(
    /*! [in] Directory pathname. */
    const char *path
);

/*!
** \ingroup file_system
** \brief Read a directory.
**
** Reads a directory stream \a dir at its current position.
** The directory stream is positioned at the next entry.
**
** \retval 0 Success.
** \retval -1 Error, see \c errno.
*/
int readdir_r(
    /*! [in,out] An object of type \c DIR to be read. */
    DIR *dir,
    /*! [out] Structure to receive the information. */
    struct dirent *dent
);

/* NOTE: Function readdir is not supported. */

/*!
** \ingroup file_system
** \brief Close a directory stream.
**
** Closes a directory designated by \a dir.
** If a file descriptor has been extracted by dirfd(), it is also closed.
**
** \retval 0 Success.
** \retval -1 Error, see \c errno.
*/
int closedir(
    /*! [in,out] An object of type \c DIR to be closed. */
    DIR *dir
);

#endif /* _LTRX_DIRENT_H_ */
